#import os
#import binascii

# import jwt
# >>> encoded = jwt.encode({"some": "payload"}, "secret", algorithm="HS256")
# >>> print(encoded)
# eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJzb21lIjoicGF5bG9hZCJ9.4twFt5NiznN84AWoo1d7KO1T_yoc0Z6XOpOVswacPZg
# >>> jwt.decode(encoded, "secret", algorithms=["HS256"])
# {'some': 'payload'}

# JWT_ALGORITHM=
# JWT_SECRET=binascii.hexlify(os.urandom(24))

# def token_response(token: str):
#     return {
#         "access_token": token
#     }

# def signJWT(user_id: str) -> Dict[str, str]:
#     payload = {
#         "user_id": user_id,
#         "expires": time.time() + 600
#     }
#     token = jwt.encode(payload, JWT_SECRET, algorithm=JWT_ALGORITHM)

#     return token_response(token)


# def decodeJWT(token: str) -> dict:
#     try:
#         decoded_token = jwt.decode(token, JWT_SECRET, algorithms=[JWT_ALGORITHM])
#         return decoded_token if decoded_token["expires"] >= time.time() else None
#     except:
#         return {}


import time
import traceback

from typing import Callable, List

from fastapi import Body, FastAPI, HTTPException, Request, Response
from fastapi.responses import JSONResponse
from fastapi.exceptions import RequestValidationError
from fastapi.routing import APIRoute


# class StandardResponseRoute(APIRoute):
#     def get_route_handler(self) -> Callable:
#         original_route_handler = super().get_route_handler()

#         async def custom_route_handler(request: Request) -> Response:
#             before = time.time()
#             try:
#                 response: Response = await original_route_handler(request)
#             except RequestValidationError as exc:
#                 detail = {
#                     "errors": exc.errors(),
#                 }
#                 raise HTTPException(status_code=422, detail=detail)
#             except Exception as exc:
#                 tb = traceback.format_tb(exc.__traceback__)
#                 detail = {
#                     "message": str(exc),
#                     "debug": {
#                         "loc": tb[-1],
#                         "stack": tb,
#                     }
#                 }
#                 raise HTTPException(status_code=500, detail=detail)
    
#             duration = 1000.0 * (time.time() - before)
#             response.headers["X-Response-Time-MS"] = str(duration)

#             return response

#         return custom_route_handler

class StandardResponseRoute(APIRoute):
    def get_route_handler(self) -> Callable:
        original_route_handler = super().get_route_handler()

        async def custom_route_handler(request: Request) -> Response:
            before = time.time()
            try:
                original_response: Response = await original_route_handler(request)
            except RequestValidationError as exc:
                detail = {
                    "errors": exc.errors(),
                }
                raise HTTPException(status_code=422, detail=detail)
            except HTTPException as exc:
                raise exc
            except Exception as exc:
                tb = traceback.format_tb(exc.__traceback__)
                response_data = {
                    'success': False,
                    'result': None,
                    'errors': [
                        {
                            "msg": str(exc),
                            "debug": {
                                "loc": tb[-1],
                                "stack": tb,
                            }
                        }
                    ]
                }
                wrapped_response = JSONResponse(response_data)
                return wrapped_response

            duration = 1000.0 * (time.time() - before)
            response.headers["X-Response-Time-MS"] = str(duration)
            return original_response

        return custom_route_handler


from fastapi import Security, Depends, HTTPException, status
from fastapi.security import HTTPBearer, HTTPAuthorizationCredentials
from fastapi.security.api_key import APIKey, APIKeyHeader

temp_api_keys = [
    "69"
]

api_key_header = APIKeyHeader(name="authorization", auto_error=False)

async def get_api_key(api_key_raw: str = Security(api_key_header)):
    try:
        (key_type, api_key) = api_key_raw.split()
        assert(key_type.lower() == 'bearer')
        assert(api_key)
    except:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN, detail="Malformed API KEY. Expecting: Authorization: Bearer {token})"
        )

    if api_key in temp_api_keys:
        return api_key_header   
    else:
        raise HTTPException(
            status_code=status.HTTP_403_FORBIDDEN, detail="Could not validate API KEY."
        )


async def get_user_id(api_key: APIKey=Depends(get_api_key)) -> int:
    return 11


#def get_user(api_key: str = Depends(oauth2_scheme)):
#    pass


# from .jwt import decodeJWT


# class ApiTokenBearer(HTTPBearer):
#     def __init__(self, auto_error: bool = True):
#         super(ApiTokenBearer, self).__init__(auto_error=auto_error)

#     async def __call__(self, request: Request):
#         credentials: HTTPAuthorizationCredentials = await super(ApiTokenBearer, self).__call__(request)
#         print('CREDS 4', credentials)
#         if credentials:
#             if not credentials.scheme == "Bearer":
#                 raise HTTPException(status_code=403, detail="Invalid authentication scheme.")
#         return credentials.credentials

# #     async def __call__(self, request: Request):
# #         credentials: HTTPAuthorizationCredentials = await super(ApiTokenBearer, self).__call__(request)
# #         if credentials:
# #             if not credentials.scheme == "Bearer":
# #                 raise HTTPException(status_code=403, detail="Invalid authentication scheme.")
# #             if not self.verify_jwt(credentials.credentials):
# #                 raise HTTPException(status_code=403, detail="Invalid token or expired token.")
# #             return credentials.credentials
# #         else:
# #             raise HTTPException(status_code=403, detail="Invalid authorization code.")

# #     def verify_jwt(self, jwtoken: str) -> bool:
# #         isTokenValid: bool = False

# #         try:
# #             payload = decodeJWT(jwtoken)
# #         except:
# #             payload = None
# #         if payload:
# #             isTokenValid = True
# #         return isTokenValid